####################################################################################
#
#  This file generates test data for the least-squares estimation of the incidence
#
#  Supplement to BMC Medical Research Methodology paper by Brinks R et al. for use in the 
#    statistical software R (The R Foundation of Statistical Computing)
#
#  General note: the calendar time t in counted in years after 01.01.2004 (t=0)
#
####################################################################################

rm(list=ls(all=TRUE))
source("rates.R")
library(deSolve)


####################################################################################
#
#  Calculate prevalence, general mortality and relative mortality 
#  at t = 1995, 1998, 2001, 2004 at specified ages (e.ages)
#
####################################################################################

e.ages  <- 30+2*0:35             # considered age range 30, 35, 40, ..., 75, 80 (regular age sampling, Test data A)
e.years <- seq(-9, 1, by = 1)    # t = 1995, 1998, 2001, 2004


eps.A <- -0.05
eps.B <-  0.15


png("./fig/Lambdas.png", width=1800, height=900, pointsize=10, res=200)

par(mfrow = c(1, 2))
matplot(0:100, sapply(0:100, FUN = get_l0, t = -9), type = "l", ylab = expression(lambda[0]), main="", xlab = "Age a (years)", col = "blue", lwd = 2)
matplot(0:100, sapply(0:100, FUN = get_l1, t = -9, eps = 0), type = "l", ylab = expression(lambda[1]), main="", xlab = "Age a (years)", col = "blue", lwd = 2)

dev.off()



#these are the data matrices
p1Mat.A <- matrix(data = NA, nrow = length(e.years), ncol = length(e.ages)) #prevalence from the register
p2Mat.A <- matrix(data = NA, nrow = length(e.years), ncol = length(e.ages)) 

p1Mat.B <- matrix(data = NA, nrow = length(e.years), ncol = length(e.ages)) #prevalence from the register
p2Mat.B <- matrix(data = NA, nrow = length(e.years), ncol = length(e.ages)) 

# here the calculation is done
for(tt in 1:length(e.years)){
   pp <- sapply(e.ages, FUN = fct_p,  t = e.years[tt], eps=eps.A)  #prevalences from year -9 (1995, tt = 1) to 1 (2005, tt = 10)
   p1Mat.A[tt, ] <- pp[1, ]
   p2Mat.A[tt, ] <- pp[2, ]

   pp <- sapply(e.ages, FUN = fct_p,  t = e.years[tt], eps=eps.B)  
   p1Mat.B[tt, ] <- pp[1, ]
   p2Mat.B[tt, ] <- pp[2, ]
}

par(mfrow=c(1,2))
matplot(e.ages,  p1Mat.A[1,  ], type="l", xlab = "Age (years)", ylab = "Prevalence", ylim=c(0,0.17), col="red", lwd=2)
matlines(e.ages, p2Mat.A[1,  ], col="orange", lwd=2)
matlines(e.ages, p1Mat.A[6,  ], lty=1, col="red")
matlines(e.ages, p2Mat.A[6,  ], lty=1, col="orange")
matlines(e.ages, p1Mat.A[11, ], lty=2, col="red")
matlines(e.ages, p2Mat.A[11, ], lty=2, col="orange")

matplot(e.ages,  p1Mat.B[1,  ], type="l", xlab = "Age (years)", ylab = "Prevalence", ylim=c(0,0.17), col="red", lwd=2)
matlines(e.ages, p2Mat.B[1,  ], col="orange", lwd=2)
matlines(e.ages, p1Mat.B[6,  ], lty=1, col="red")
matlines(e.ages, p2Mat.B[6,  ], lty=1, col="orange")
matlines(e.ages, p1Mat.B[11, ], lty=2, col="red")
matlines(e.ages, p2Mat.B[11, ], lty=2, col="orange")

# Prevalence ratios wp1, wp2
wp1a <- p1Mat.A/(p1Mat.A + p2Mat.A)
wp2a <- p2Mat.A/(      1 - p2Mat.A)
wp1b <- p2Mat.B/(p1Mat.B + p2Mat.B)
wp2b <- p1Mat.B/(      1 - p2Mat.B)


png("./fig/Prev_A.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, p1Mat.A[11, ], type="l", col="red", lty = 2, lwd = 2, 
          xlab="Age a (years)", ylab=expression(p[1]), ylim=c(0, 0.16), main="")
matplot(e.ages, p1Mat.A[6, ], type="l", col="red", lty = 1, lwd = 2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"),
                           lty = c(2, 1), col = "red", lwd = 2)

matplot(e.ages, p2Mat.A[11, ], type="l", col="orange", lty = 2, lwd = 2,
          xlab="Age a (years)", ylab=expression(p[2]), ylim=c(0, 0.16), main = "")
matplot(e.ages, p2Mat.A[6, ], type="l", col="orange", lty = 1, lwd = 2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"),
                           lty = c(2, 1), col = "orange", lwd = 2)

dev.off()


png("./fig/Prev_B.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, p1Mat.B[11, ], type="l", col="red", lty = 2, lwd = 2, 
          xlab="Age a (years)", ylab=expression(p[1]), ylim=c(0, 0.16), main="")
matplot(e.ages, p1Mat.B[6, ], type="l", col="red", lty = 1, lwd = 2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"),
                           lty = c(2, 1), col = "red", lwd = 2)

matplot(e.ages, p2Mat.B[11, ], type="l", col="orange", lty = 2, lwd = 2,
          xlab="Age a (years)", ylab=expression(p[2]), ylim=c(0, 0.16), main = "")
matplot(e.ages, p2Mat.B[ 6, ], type="l", col="orange", lty = 1, lwd = 2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"),
                           lty = c(2, 1), col = "orange", lwd = 2)

dev.off()



png("./fig/PR_A.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
PR1.1 <- wp1a[6,  ]
PR1.2 <- wp1a[11, ]

matplot(e.ages, PR1.2, type = "l", lty = 2, xlab = "Age a (years)", ylab = expression(p[2]/(p[1]+p[2])),
           main="", col="blue", lwd=2, ylim=c(0, 1))
matplot(e.ages, PR1.1, type = "l", lty = 1, add = TRUE, col="blue", lwd=2)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2,1), col="blue", lwd=2)

PR2.1 <- wp2a[6,  ]
PR2.2 <- wp2a[11, ]

matplot(e.ages, PR2.2, type = "l", lty = 2, xlab = "Age a (years)", ylab = expression(p[1]/(p[0]+p[1])),
           main="", col="blue", lwd=2, ylim=c(0, 0.17))
matplot(e.ages, PR2.1, type = "l", lty = 1, add = TRUE, col="blue", lwd=2)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd=2,  col="blue")

dev.off()


png("./fig/PR_B.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
PR1.1 <- wp1b[6,  ]
PR1.2 <- wp1b[11, ]

matplot(e.ages, PR1.2, type = "l", lty = 2, xlab = "Age a (years)", ylab = expression(p[2]/(p[1]+p[2])),
           main="", col="blue", lwd=2, ylim=c(0, 1))
matplot(e.ages, PR1.1, type = "l", lty = 1, add = TRUE, col="blue", lwd=2)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2,1), col="blue", lwd=2)

PR2.1 <- wp2b[6, ]
PR2.2 <- wp2b[11, ]

matplot(e.ages, PR2.2, type = "l", lty = 2, xlab = "Age a (years)", ylab = expression(p[1]/(p[0]+p[1])),
           main="", col="blue", lwd=2, ylim=c(0, 0.07))
matplot(e.ages, PR2.1, type = "l", lty = 1, add = TRUE, col="blue", lwd=2)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd=2,  col="blue")

dev.off()


## Figure 
g_ <- 0
DR0.1 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ - 4, eps = eps.A)/sapply(e.ages, FUN = get_l0, t = -4)
DR0.2 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ + 1, eps = eps.A)/sapply(e.ages, FUN = get_l0, t =  1)
g_ <- 5
DR5.1 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ - 4, eps = eps.A)/sapply(e.ages, FUN = get_l0, t = -4)
DR5.2 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ + 1, eps = eps.A)/sapply(e.ages, FUN = get_l0, t =  1)


png("./fig/DR_A.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, DR0.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(5, 500), 
        ylab = expression(DR[0]), main="", col="blue", lwd=2, log="y")
matplot(e.ages, DR0.2, type = "l", lty = 2, col="blue", lwd=2, log="y", add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

matplot(e.ages, DR5.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(5, 500), 
        ylab = expression(DR[5]), main="", lwd = 2, col="blue", log="y")
matplot(e.ages, DR5.2, type = "l", lty = 2, lwd = 2, col="blue", log="y", add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

dev.off()



fct_lm <- function(u, tt, aa, ee){
   return(get_l1(tt+u, aa+u, ee) + get_m1(tt+u, aa+u))
}

fct_exp_lm <- function(s, t, a, eps){
  return(pracma::romberg(fct_lm, 0, s, tt=t, aa=a, ee=eps, tol=1e-8)$value)
}

fct_int1 <- function(s, tt, aa, ee){
  return(get_l1(tt+s, aa+s, ee)*exp(-fct_exp_lm(s, tt, aa, ee)))
}

fct_int2 <- function(s, tt, aa, ee){
  return(get_m1(tt+s, aa+s)*exp(-fct_exp_lm(s, tt, aa, ee)))
}

P_detect <- function(t, a, gamma, eps){
   return(pracma::romberg(fct_int1, 0, gamma, tt=t, aa=a, ee = eps, tol=1e-8)$value)
}

P_dead  <- function(t, a, gamma, eps){
   return(pracma::romberg(fct_int2, 0, gamma, tt=t, aa=a, ee = eps, tol=1e-8)$value)
}




ic1  <- 1e5*sapply(e.ages, FUN = get_l0, t = -4)
ic2  <- 1e5*sapply(e.ages, FUN = get_l0, t =  1)

pd1a <- sapply(e.ages, FUN = P_dead, t = -4, gamma = 5, eps = eps.A)
pd2a <- sapply(e.ages, FUN = P_dead, t =  1, gamma = 5, eps = eps.A)


CDR.1 <- wp2a[6,  ] * sapply(e.ages, FUN = get_l1, t = -4, eps = eps.A)/sapply(e.ages, FUN = get_l0, t = -4)
CDR.2 <- wp2a[11, ] * sapply(e.ages, FUN = get_l1, t =  1, eps = eps.A)/sapply(e.ages, FUN = get_l0, t =  1)

png("./fig/CDR_Phi_A.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, ic1*pd1a, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(0, 1.2e3),
        ylab = expression(Phi[5] ~ "(per 100,000)"), main="", col="blue", lwd=2)
matplot(e.ages, ic2*pd2a, type = "l", lty = 2, col="blue", lwd=2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

matplot(e.ages, CDR.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(0, 2),
        ylab = expression(CDR), main="", col="blue", lwd=2)
matplot(e.ages, CDR.2, type = "l", lty = 2, col="blue", lwd=2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

dev.off()



## Figure 9
g_ <- 0
DR0.1 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ - 4, eps = eps.B)/sapply(e.ages, FUN = get_l0, t = -4)
DR0.2 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ + 1, eps = eps.B)/sapply(e.ages, FUN = get_l0, t =  1)
g_ <- 5
DR5.1 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ - 4, eps = eps.B)/sapply(e.ages, FUN = get_l0, t = -4)
DR5.2 <- sapply(g_ + e.ages, FUN = get_l1, t = g_ + 1, eps = eps.B)/sapply(e.ages, FUN = get_l0, t =  1)

png("./fig/DR_B.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, DR0.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(1.5, 1500), 
        ylab = expression(DR[0]), main="", col="blue", lwd=2, log="y")
matplot(e.ages, DR0.2, type = "l", lty = 2, col="blue", lwd=2, log="y", add = TRUE)
legend("topright", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

matplot(e.ages, DR5.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(1.5, 1500), 
        ylab = expression(DR[5]), main="", lwd = 2, col="blue", log="y")
matplot(e.ages, DR5.2, type = "l", lty = 2, lwd = 2, col="blue", log="y", add = TRUE)
legend("topright", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

dev.off()


## Figure 10

ic1  <- 1e5*sapply(e.ages, FUN = get_l0, t = -4)
ic2  <- 1e5*sapply(e.ages, FUN = get_l0, t =  1)

pd1b <- sapply(e.ages, FUN = P_dead, t = -4, gamma = 5, eps = eps.B)
pd2b <- sapply(e.ages, FUN = P_dead, t =  1, gamma = 5, eps = eps.B)


CDR.1 <- wp2b[6,  ] * sapply(e.ages, FUN = get_l1, t = -4, eps = eps.B)/sapply(e.ages, FUN = get_l0, t = -4)
CDR.2 <- wp2b[11, ] * sapply(e.ages, FUN = get_l1, t =  1, eps = eps.B)/sapply(e.ages, FUN = get_l0, t =  1)

png("./fig/CDR_Phi_B.png", width=1800, height=900, pointsize=10, res=200)
par(mfrow = c(1, 2))
matplot(e.ages, ic1*pd1b, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(0, 1.2e3),
        ylab = expression(Phi[5] ~ "(per 100,000)"), main="", col="blue", lwd=2)
matplot(e.ages, ic2*pd2b, type = "l", lty = 2, col="blue", lwd=2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

matplot(e.ages, CDR.1, type = "l", lty = 1, xlab = "Age a (years)", ylim=c(0, 2),
        ylab = expression(CDR), main="", col="blue", lwd=2)
matplot(e.ages, CDR.2, type = "l", lty = 2, col="blue", lwd=2, add = TRUE)
legend("topleft", legend=c("Year 2005", "Year 2000"), lty = c(2, 1), lwd = 2, col="blue")

dev.off()


#Table 4
ergMat <- matrix(NA, 4, 6)
ages <- 45+15*0:3
for(ii in 1:4){
  aa <- ages[ii]
  ic1  <- 1e5*get_l0(-4, aa) # * (1-prev1-prev2)[101, aa+1])
  ic2  <- 1e5*get_l0( 1, aa) # * (1-prev1-prev2)[101, aa+1])
  pd1a <- P_dead(-4, aa, 5, eps.A)
  pd2a <- P_dead( 1, aa, 5, eps.A)
  pd1b <- P_dead(-4, aa, 5, eps.B)
  pd2b <- P_dead( 1, aa, 5, eps.B)
  ergMat[ii, ] <- c(ic1, ic2, ic1*pd1a, ic2*pd2a, ic1*pd1b, ic2*pd2b)
}
ergMat

