####################################################################################
#
#  This file interfaces the data from the Danish National Diabetes Register
#  as published by Carstensen et al. in Diabetologia 2008, 51(12):2187-96
#  DOI 10.1007/s00125-008-1156-z
#
#  Supplement to BMC Medical Research Methodology paper by Brinks R et al. for use in the 
#    statistical software R (The R Foundation of Statistical Computing)
#
#  General note: the calendar time t in counted in years after 01.01.2004 (t=0)
#
####################################################################################


#Incidence data 2004 for males, Figure 1 in Carstensen et al. 2008
inc.ages <- 10*3:10
inc.data <- 1e-3*c(0.65, 1.95, 4.5, 8.5, 11.2, 11.7, 9.5, 6.3)

#Mortality data 2004, Figure 4, leftmost part
m.ages   <- 10*5:8
m0.data  <- 1e-3*c(4.2, 10.1, 29, 77) #non-diabetic
m1.data  <- 1e-3*c(17, 31, 64, 150)   #diabetic

#inc is modelled log-parabolic
inc.coeff <- c(-12.207363899, 0.201429986, -0.001302584) # calculated by lm(log(inc.data) ~ I(inc.ages) + I(inc.ages^2))$coefficients

#mort is modelled log-linear
m0.coeff  <- c(-10.40067499, 0.09780923) # calculated by lm(log(m0.data)  ~ I(m.ages))$coefficients
m1.coeff  <- c( -7.76573067, 0.07257162) # calculated by lm(log(m1.data)  ~ I(m.ages))$coefficients

#generic rate function
fct_rate <- function(t, a, aCoeff, tRate = 1){
  logTrendRate <- log(tRate)
  if(length(aCoeff) == 2)
     logARate <- aCoeff[1] + a*aCoeff[2]
  if(length(aCoeff) == 3)
     logARate <- aCoeff[1] + a*(aCoeff[2] + aCoeff[3]*a)
  return(as.numeric(exp(t*logTrendRate + logARate)))
}

# Incidence rate in the male population, temporal trends as reported in Carstensen et al. 2008
get_l0  <- function(t, a){
#   return(1.25*fct_rate(t, a, aCoeff = inc.coeff, tRate = 1.053))
   return(2.0*fct_rate(t, a, aCoeff = inc.coeff, tRate = 1.053))
}

get_l1 <- function(t, a, eps = 0){
#   aa <- 40
   aa <- 30
   cc <- 0.3
   bb <- 0.25
   return(cc*exp(max(0, t+4)*log(1.0 + eps))/(1+exp(-bb*(a-aa))))
}


# Mortality rate in the non-diabetic male population
get_m0 <- function(t, a){
   return(fct_rate(t, a, aCoeff = m0.coeff, tRate = 0.975))
}

# Mortality rate in the undiagnosed-diabetic male population
get_m1 <- function(t, a){
   return(1.25*fct_rate(t, a, aCoeff = m0.coeff, tRate = 0.975))
}

# Mortality rate in the diabetic male population
get_m2 <- function(t, a){
   return(fct_rate(t, a, aCoeff = m1.coeff, tRate = 0.961))
}

# Prevalence of diabetes in the male population at 01.01.1995 (t = -9)
# Carstensen et al., 2008, Figure 2, leftmost part
get_p2 <- function(a){
    a_ <- c(19.6, 39.5, 58.3, 65.1, 69.6, 75.6, 80.0, 83.5, 87.6 , 97.7) # ages
    p_ <- c( 0.4,  1.1,  4.0,  5.4,  6.0,  7.2,  7.5,  7.8,  7.3,   5.1) # prev
    spline <- smooth.spline(a_, p_, df=6)
    return(0.01*predict(spline, a)$y)
}

get_p1 <- function(a){
    return(0.5*get_p2(a))
}


# right hand side of the differential equation
rhs <- function(t, y, parms) 
{
   with(as.list(parms), 
   {
      m0_  <- get_m0(t, a0 - ts + t)
      m1_  <- get_m1(t, a0 - ts + t)
      m2_  <- get_m2(t, a0 - ts + t)
      l0_  <- get_l0(t, a0 - ts + t)
      l1_  <- get_l1(t, a0 - ts + t, eps)
      p1_ <- y[1]
      p2_ <- y[2]

      # next line is the differential equation
      zz <- (m1_-m0_)*p1_ + (m2_-m0_)*p2_
      dy <- c(l0_ - p1_*(l1_+l0_+m1_-m0_) - p2_*l0_ + p1_*zz,
                    p1_* l1_  - p2_*(m2_-m0_)       + p2_*zz
              )
      list(dy)
   }   )
}

# calculates the prevalence of those aged a at time t
fct_p <- function(t, a, eps=0){
  ts         <- -9                         #prevalence at 01.01.1995 -> ts = -9
  as         <- a - (t-ts)                 # age at t, probe: t = -9 -> as = a, t = 0 -> as = a + 9
  parms      <- c(a0 = as, ts = ts, eps = eps)
  y_         <- c(get_p1(as), get_p2(as))  #initial condition
  t_         <- seq(ts, t, by=0.5)         #which as to be used
  x          <- c(p = y_)
  
  #now solve the differential equation
  out_       <- as.data.frame(deSolve::rk4(x, t_, rhs, parms))

  return(c(out_$p1[length(t_)], out_$p2[length(t_)]))
}
